#!/usr/bin/env python3
import argparse
import os.path
import sys
from signal import signal, SIGINT

import requests
import urllib3

API_HOST = 'apis.uloz.to'
API_TOKEN = ';HG%7jW6@6/8vx">R;f('


# Parse script parameters
parser = argparse.ArgumentParser()
parser.add_argument('files', help='Files to be uploaded', nargs=argparse.REMAINDER)
parser.add_argument('-l', '--login', help='User account login at uloz.to website', required=True)
parser.add_argument('--folder-slug', help='Slug of target folder. Uploaded files will be stored in the folder with'
                                          ' this slug. If none is given files will be stored in the'
                                          ' default root folder.')
parser.add_argument('--verbose', help='Increase verbosity.', action='store_true')
parser.add_argument('-t', '--api-token', help='Unique application token. Can be obtained from uloz.to support.',
                    default=API_TOKEN)
parser.add_argument('--auth-token', help='Unique authentication token to be used instead of password. Can be obtained from uloz.to web.',
                    required=True)
parser.add_argument('--retry-slug', help='Private slug of an unfinished batch.')
parser.add_argument('--api-host', help='Ulozto/Pornfile API hostname.', default=API_HOST)
parser.add_argument('--ignore-ssl', help='Ignore SSL verification in requests.', action='store_true')
args = parser.parse_args()

if not args.files:
    print('No files to upload')
    sys.exit(0)


def print_request_verbose(response, print_request_body=True):
    if not args.verbose:
        return

    request = response.request
    request_headers_str = ', '.join(f'{key}:{val}' for key, val in request.headers.items())
    request_items = [f'{request.method} {request.url}', f'Headers: {request_headers_str}']
    if print_request_body:
        request_body = request.body.decode() if isinstance(request.body, bytes) else str(request.body)
        request_items.append(f'Body: {request_body}')
    request_str = 'Request: {}'.format('\n'.join(request_items))

    response_headers_str = ', '.join(f'{key}:{val}' for key, val in response.headers.items())
    response_str = f'Response: {response.status_code}\nHeaders: {response_headers_str}\nBody: {response.text}'
    print(f'{request_str}\n{response_str}')


def verbose_line_break():
    if args.verbose:
        print()


def rebuild_script_command_with_retry_args():
    if is_rerun or nothing_uploaded:
        additional_args = []
    else:
        additional_args = ['--retry-slug', private_slug]
    input_files_start = len(sys.argv) - len(args.files)
    quoted_input_files = [f'"{file}"' for file in sys.argv[input_files_start:]]
    return ' '.join((sys.argv[0], *additional_args, *sys.argv[1:input_files_start], *quoted_input_files))


def sigint_handler(*_):
    if private_slug:
        print('Upload script interrupted')
        print(f'To continue, try running:\n{rebuild_script_command_with_retry_args()}')
    sys.exit(0)


signal(SIGINT, sigint_handler)

api_url = f'https://{args.api_host}'
private_slug = None
nothing_uploaded = True
is_rerun = False
if args.retry_slug:
    is_rerun = True
should_verify = True
if args.ignore_ssl:
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    should_verify = False


# Set up session
session = requests.Session()
session.headers = {
    "X-Auth-Token": args.api_token,
    "Content-type": "application/json",
    "Accept": "application/json",
}


# Get user token
print(f"User {args.login} authentication")
login = {"login": args.login, "token": args.auth_token}
user_token_response = session.post(url=f'{api_url}/v5/auth/token', json=login, verify=should_verify)
if user_token_response.status_code == 400:
    print(f'Invalid authentication token or username!')
    sys.exit(1)
user_token_response.raise_for_status()

session.headers["X-User-Token"] = user_token_response.json()["token_id"]
print('Authentication successful')
verbose_line_break()


# Get/Refresh upload link
print('Fetching upload link')
user = {"user_login": args.login, 'private_slug': args.retry_slug}
upload_link_response = session.post(url=f"{api_url}/v6/upload/link", json=user, verify=should_verify)
print_request_verbose(upload_link_response)
upload_link_response.raise_for_status()

private_slug = upload_link_response.json()['private_slug']
upload_url_base = upload_link_response.json()['upload_url']
print('Upload link fetched')
verbose_line_break()

if is_rerun:
    print('Retrieving completed file ids')
    view_batch_response = session.get(url=f'{api_url}/v8/upload-batch/private/{private_slug}/file-list', verify=should_verify)
    print_request_verbose(view_batch_response)
    view_batch_response.raise_for_status()
    verbose_line_break()
    completed_files = {item['batch_file_id']: item['file']['slug'] for item in view_batch_response.json()['items']}

try:
    # Upload files
    files_slugs_dict = dict()
    upload_token_dictionary = {}
    for batch_file_id, file_to_upload in enumerate(args.files, start=1):
        file_name = os.path.basename(file_to_upload)
        if is_rerun and batch_file_id in completed_files.keys():
            file_slug = completed_files[batch_file_id]
            files_slugs_dict[file_slug] = file_name
            upload_token_dictionary[file_slug] = f'{private_slug}:{batch_file_id}'
            continue

        print(f'Uploading file {batch_file_id} "{file_name}"')
        upload_url = f"{upload_url_base}&batch_file_id={batch_file_id}"
        upload_response = requests.post(url=upload_url, files={'file': open(file_to_upload, 'rb')}, verify=should_verify)
        print_request_verbose(upload_response, print_request_body=False)
        upload_response.raise_for_status()

        nothing_uploaded = False
        file_slug = upload_response.json()['slug']
        files_slugs_dict[file_slug] = file_name
        upload_token_dictionary[file_slug] = f'{private_slug}:{batch_file_id}'
        print('Upload successful')
        verbose_line_break()

    # Set file properties
    print('Setting file properties')
    batch_properties = {'slugs': list(files_slugs_dict.keys()),
                        'upload_tokens': upload_token_dictionary,
                        }
    if args.folder_slug:
        batch_properties['folder_slug'] = args.folder_slug
    else:
        session_response = session.get(f"{api_url}/v5/me", verify=should_verify)
        print_request_verbose(session_response)
        session_response.raise_for_status()
        verbose_line_break()
        batch_properties['folder_slug'] = session_response.json()['user']['root_folder_slug']

    properties_response = session.patch(url=f"{api_url}/v8/file-list/private", json=batch_properties, verify=should_verify)
    print_request_verbose(properties_response)
    properties_response.raise_for_status()
    print("File properties set")
    verbose_line_break()

    # Commit batch
    print('Committing files')
    commit = {'status': 'confirmed', 'owner_login': args.login}
    commit_batch_response = session.patch(url=f"{api_url}/v8/upload-batch/private/{private_slug}", json=commit, verify=should_verify)
    print_request_verbose(commit_batch_response)
    commit_batch_response.raise_for_status()
    verbose_line_break()

    site = api_url.split('.', 1)[1]
    print(f'Files committed:')
    for file_slug, file_path in files_slugs_dict.items():
        print(f'https://{site}/file/{file_slug}/{file_path}')

except Exception as exc:
    print(f'Upload interrupted by: {exc}')
    print(f'To continue, try running:\n{rebuild_script_command_with_retry_args()}')
    sys.exit(1)
